#ifndef __X264_GTK_ENUM_H__
#define __X264_GTK_ENUM_H__

typedef enum
{
  X264_PASS_SINGLE_BITRATE,
  X264_PASS_SINGLE_QUANTIZER,
  X264_PASS_MULTIPASS_1ST,
  X264_PASS_MULTIPASS_1ST_FAST,
  X264_PASS_MULTIPASS_NTH
}X264_Pass;

typedef enum
{
  X264_NONE     = X264_DIRECT_PRED_NONE,
  X264_SPATIAL  = X264_DIRECT_PRED_SPATIAL,
  X264_TEMPORAL = X264_DIRECT_PRED_TEMPORAL,
  X264_AUTO     = X264_DIRECT_PRED_AUTO
}X264_Direct_Mode;

typedef enum
{
  X264_PD_1,
  X264_PD_2,
  X264_PD_3,
  X264_PD_4,
  X264_PD_5,
  X264_PD_6,
  X264_PD_6b
}X264_Partition_Decision;

typedef enum
{
  X264_ME_METHOD_DIAMOND          = X264_ME_DIA,
  X264_ME_METHOD_HEXAGONAL        = X264_ME_HEX,
  X264_ME_METHOD_UNEVEN_MULTIHEXA = X264_ME_UMH,
  X264_ME_METHOD_EXHAUSTIVE       = X264_ME_ESA
}X264_Me_Method;

typedef enum
{
  X264_DEBUG_METHOD_NONE    = X264_LOG_NONE + 1,
  X264_DEBUG_METHOD_ERROR   = X264_LOG_ERROR + 1,
  X264_DEBUG_METHOD_WARNING = X264_LOG_WARNING + 1,
  X264_DEBUG_METHOD_INFO    = X264_LOG_INFO + 1,
  X264_DEBUG_METHOD_DEBUG   = X264_LOG_DEBUG + 1
}X264_Debug_Method;

typedef enum
{
  X264_CQM_PRESET_FLAT   = X264_CQM_FLAT,
  X264_CQM_PRESET_JVT    = X264_CQM_JVT,
  X264_CQM_PRESET_CUSTOM = X264_CQM_CUSTOM
}X264_Cqm_Preset;


#endif /* __X264_GTK_ENUM_H__ */
